
/*
    provide commands for access to the AGATA digitizer via the Xport interface (with Dave Wells' firmware)

*/

#ifdef WIN32
#include <windows.h>
#else
#include <sys/types.h>
#endif

#include <stdlib.h>
#include <stdio.h>

#include <tcl.h>

#ifdef WIN32
#define uint32_t unsigned int
#define int32_t int
#define uint16_t unsigned short
#define uint8_t  char

#endif

#include "transfer.h"

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_InitCmd _ANSI_ARGS_((ClientData clientdata, Tcl_Interp *interp, int argc, char * argv[]));

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_SelectCmd _ANSI_ARGS_((ClientData clientdata, Tcl_Interp *interp, int argc, char * argv[]));

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_VerboseCmd _ANSI_ARGS_((ClientData clientdata, Tcl_Interp *interp, int argc, char * argv[]));

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_ConfigureCmd _ANSI_ARGS_((ClientData clientdata, Tcl_Interp *interp, int argc, char * argv[]));

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_GetCmd _ANSI_ARGS_((ClientData clientdata, Tcl_Interp *interp, int argc, char * argv[]));

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_SetCmd _ANSI_ARGS_((ClientData clientdata, Tcl_Interp *interp, int argc, char * argv[]));

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_WriteCmd _ANSI_ARGS_((ClientData clientdata, Tcl_Interp *interp, int argc, char * argv[]));


int Xport_Verbose = 0;
int Xport_CurrentUser = 0;

char Xport_response[8192];
unsigned char Xport_databuffer[1100];

char *Xport_Device = (char*) "localhost";
int Xport_Port = 10001;
int Xport_BufferSize = 1024;
int Xport_SndBufSize = 512;

int Xport_SendTimeout = 5000;
int Xport_RecvTimeout = 5000;


#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_VerboseCmd(ClientData cdata, Tcl_Interp *interp, int argc, char * argv[])
{
    int i;

    for (i=1;i<argc;i++) {
        switch (i) {
           case 1:
              Xport_Verbose = atoi(argv[i]);   
           break;
           default:
           break;
        }
    }

    sprintf(Xport_response, "%d", 0);

    Tcl_SetResult (interp, Xport_response, TCL_VOLATILE);

    return TCL_OK;
}

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_SelectCmd(ClientData cdata, Tcl_Interp *interp, int argc, char * argv[])
{
    int i;

    for (i=1;i<argc;i++) {
        switch (i) {
           case 1:
              Xport_CurrentUser = atoi(argv[i]);   
           break;
           default:
           break;
        }
    }

    (void) XportSetUser(Xport_CurrentUser);

    sprintf(Xport_response, "%d", 0);

    Tcl_SetResult (interp, Xport_response, TCL_VOLATILE);

    return TCL_OK;
}


#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_ConfigureCmd(ClientData cdata, Tcl_Interp *interp, int argc, char * argv[])
{
    int i;

    if (argc >1) {
        for(i=1;i <argc;i++) {
            if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
                switch(tolower(argv[i][1])) {
                    case 'n':
                        Xport_Device = argv[++i];
                        break;
                    case 'p':
                        Xport_Port = atoi(argv[++i]);
                        break;
                    case 's':
                        Xport_BufferSize = atoi(argv[++i])*1024;
                        break;
                    case 'w':
                        Xport_SndBufSize = atoi(argv[++i]);
                        break;
                    case 't':
                        Xport_SendTimeout = atoi(argv[++i]);
                        if (Xport_SendTimeout == 0) {Xport_SendTimeout = 5000;}    /* reset to default */
                        Xport_RecvTimeout = Xport_SendTimeout;
                        (void) XportSendTimeout(Xport_SendTimeout);
                        (void) XportRecvTimeout(Xport_RecvTimeout);
                        break;
                    default:
                        Tcl_SetErrorCode(interp, "Invalid parameter in Xport_Configure", (char *)NULL);
                        return TCL_ERROR;
                }
            } else {
                 Tcl_SetErrorCode(interp, "Invalid parameter in Xport_Configure", (char *)NULL);
                 return TCL_ERROR;
            }
        }
    }

    sprintf(Xport_response, "%d", 0);

    Tcl_SetResult (interp, Xport_response, TCL_VOLATILE);

    return TCL_OK;
}

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_InitCmd(ClientData cdata, Tcl_Interp *interp, int argc, char * argv[])
{
    int rc;

    // This function performs the basic initialization of Xport communications

    printf( "Establishing TCP connection to Xport device at %s:%d ...\n", Xport_Device, Xport_Port);

    XportClose();        /*   tidy up any previous use  */

//    Initialise the transfer procedures 

    (void) XportReset();
    (void) XportSetVerbose(Xport_Verbose);
    (void) XportBlockSize(Xport_BufferSize);
    (void) XportSndBufSize(Xport_SndBufSize);
    (void) XportPort(Xport_Port);
    (void) XportSendTimeout(Xport_SendTimeout);
    (void) XportRecvTimeout(Xport_RecvTimeout);

//    Establish connection

    rc = XportInit (Xport_Device);
    if (rc != 0)            // handle case where there is no Xport Open returns error
    {
        printf("Error response returned by XportInit routine  !!!\n");
        Tcl_SetErrorCode(interp, "Error response returned by XportInit routine", (char *)NULL);  
        return TCL_ERROR;
    }

    sprintf(Xport_response, "%d", 0);

    Tcl_SetResult (interp, Xport_response, TCL_VOLATILE);

    return TCL_OK;
}

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_GetCmd(ClientData cdata, Tcl_Interp *interp, int argc, char * argv[])
{
    int i, l, rc;
    int len;
    int cmd;

    if (argc != 9) {Tcl_SetErrorCode(interp, "Invalid number of parameters in Xport_Get", (char *)NULL); return TCL_ERROR;}

    for (len=0,i=1;i<argc;i++) {
        Xport_databuffer[len] = (unsigned char) strtol(argv[i], (char **)NULL, 0);
        len++;
    }

    cmd = (Xport_databuffer[0] >> 5) & 3;   /* write=0; long write=1; read=2 */

    if (cmd != 2) {Tcl_SetErrorCode(interp, "Invalid command in Xport_Get", (char *)NULL); return TCL_ERROR;}

    rc = XportTxData ((unsigned char*)&Xport_databuffer[0] ,&len);  /* send the command */

    if (rc != 0) {
        sprintf(Xport_response, "Xport_Get returned 0x%x", rc);
        Tcl_SetErrorCode(interp, Xport_response, (char *)NULL);
        return TCL_ERROR;
    }

    /*     return the response      */

     for (i=0,l=0; i<len; i++) {
        rc = sprintf(&Xport_response[l], "0x%x ", Xport_databuffer[i]);
        l += rc;
     }

    Tcl_SetResult (interp, Xport_response, TCL_VOLATILE);

    return TCL_OK;
}


#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_SetCmd(ClientData cdata, Tcl_Interp *interp, int argc, char * argv[])
{
    int i, l, rc;
    int len;
    int cmd;

    if (argc != 9) {Tcl_SetErrorCode(interp, "Invalid number of parameters in Xport_Set", (char *)NULL); return TCL_ERROR;}

    for (len=0,i=1;i<argc;i++) {
        Xport_databuffer[len] = (unsigned char) strtol(argv[i], (char **)NULL, 0);
        len++;
    }

    cmd = (Xport_databuffer[0] >> 5) & 3;   /* write=0; long write=1; read=2  */

    if (cmd != 0) {Tcl_SetErrorCode(interp, "Invalid command in Xport_Set", (char *)NULL); return TCL_ERROR;}

    rc = XportTxData ((unsigned char*)&Xport_databuffer[0] ,&len);  /* send the command */

    if (rc != 0) {
        sprintf(Xport_response, "Xport_Set returned 0x%x", rc);
        Tcl_SetErrorCode(interp, Xport_response, (char *)NULL);
        return TCL_ERROR;
    }

    /*     return the response      */

    for (i=0,l=0; i<len; i++) {
        rc = sprintf(&Xport_response[l], "0x%x ", Xport_databuffer[i]);
        l += rc;
    }

    Tcl_SetResult (interp, Xport_response, TCL_VOLATILE);

    return TCL_OK;
}


#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xport_WriteCmd(ClientData cdata, Tcl_Interp *interp, int argc, char * argv[])
{
    int i, l, rc;
    int len;
    int cmd;

//    if (argc != 7) {Tcl_SetErrorCode(interp, "Invalid number of parameters in Xport_Write", (char *)NULL); return TCL_ERROR;}

    for (len=0,i=1;i<argc;i++) {
        Xport_databuffer[len] = (unsigned char) strtol(argv[i], (char **)NULL, 0);
        len++;
    }

    cmd = (Xport_databuffer[0] >> 5) & 3;   /* write=0; long write=1; read=2  */

    if (cmd != 1) {Tcl_SetErrorCode(interp, "Invalid command in Xport_Write", (char *)NULL); return TCL_ERROR;}

    rc = XportTxData ((unsigned char*)&Xport_databuffer[0] ,&len);  /* send the command */

    if (rc != 0) {
        sprintf(Xport_response, "Xport_Write returned 0x%x", rc);
        Tcl_SetErrorCode(interp, Xport_response, (char *)NULL);
        return TCL_ERROR;
    }

    /*     return the response      */

    for (i=0,l=0; i<len; i++) {
        rc = sprintf(&Xport_response[l], "0x%x ", Xport_databuffer[i]);
        l += rc;
    }

    Tcl_SetResult (interp, Xport_response, TCL_VOLATILE);

    return TCL_OK;
}



/*    
    Initialise 

*/

#ifdef WIN32
DLLEXPORT
#else
int
#endif
Xportaccess_Init (Tcl_Interp *interp)
{
   
    Tcl_CreateCommand (interp, "XportAccess_Init", (Tcl_CmdProc*) Xport_InitCmd, NULL,NULL);
    Tcl_CreateCommand (interp, "XportAccess_Select", (Tcl_CmdProc*) Xport_SelectCmd, NULL,NULL);
    Tcl_CreateCommand (interp, "XportAccess_Verbose", (Tcl_CmdProc*) Xport_VerboseCmd, NULL,NULL);
    Tcl_CreateCommand (interp, "XportAccess_Configure", (Tcl_CmdProc*) Xport_ConfigureCmd, NULL,NULL);
    Tcl_CreateCommand (interp, "XportAccess_Get", (Tcl_CmdProc*) Xport_GetCmd, NULL,NULL);
    Tcl_CreateCommand (interp, "XportAccess_Set", (Tcl_CmdProc*) Xport_SetCmd, NULL,NULL);
    Tcl_CreateCommand (interp, "XportAccess_Write", (Tcl_CmdProc*) Xport_WriteCmd, NULL,NULL);

    XportResetAll();

    printf("Xport Access package 1.0\n");
    Tcl_PkgProvide(interp, "XportAccess", "1.0");
    return TCL_OK;
}


