

#ifdef WIN32
#include <winsock2.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <time.h>

#include "transfer.h"

void Usage(char *progname) {
    fprintf(stderr,"Usage\n%s -n [server] -p [port] -s [buffersize]\n",progname);
    fprintf(stderr,"\tserver is the IP address or name of Xport device\n");
    fprintf(stderr,"\tport is the TCP port to use\n");
    fprintf(stderr,"\ts is data block size in Kbytes\n");
    fprintf(stderr,"Defaults are localhost; 10001 and 16");
    exit(1);
}
        
int main(int argc, char **argv) {

    char *Buffer;
    char *Xport = (char*) "localhost";
    int Port=10001;
    int verbose=0;
    int s;
    int BufferSize=16*1024;
    int SndBufSize=512;           // units Kbytes
    int RcvBufSize=512;  
    int i, len;
    int b;
    int NumBlocks=100;

    struct timeval Time;
    int InitialTime, EndTime;
    int diff;
    int rate;

#ifndef WIN32
    struct timespec time_request;
#endif


    if (argc >1) {
        for(i=1;i <argc;i++) {
            if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
                switch(tolower(argv[i][1])) {
                    case 'n':
                        Xport = argv[++i];
                        break;
                    case 'p':
                        Port = atoi(argv[++i]);
                        break;
                    case 's':
                        BufferSize = atoi(argv[++i])*1024;
                        break;
                    case 'w':
                        SndBufSize = atoi(argv[++i]);
                        break;
                    case 'r':
                        RcvBufSize = atoi(argv[++i]);
                        break;
                    case 'v':
                        verbose = atoi(argv[++i]);
                        break;
                    case 'b':
                        NumBlocks = atoi(argv[++i]);
                        break;
                    case '?':
                    default:
                        Usage(argv[0]);
                        break;
                }
            }
        }
    }

     Buffer = (char *)malloc(BufferSize+64);

    (void) XportSetVerbose(verbose);
    (void) XportBlockSize(BufferSize+32);
    (void) XportSndBufSize(SndBufSize);
    (void) XportRcvBufSize(RcvBufSize);
    (void) XportPort(Port);
    (void) XportInit(Xport);


#ifdef WIN32
    time(&Time.tv_sec);
#else
    gettimeofday(&Time, NULL);
#endif

    InitialTime = Time.tv_sec;
    printf("Start at %d\n", InitialTime);

//    setup command 12 

      Buffer[0] = 0xa0;
      Buffer[1] = 0;
      Buffer[2] = 0;
      Buffer[3] = 8;
      Buffer[4] = 0xb0;
      Buffer[5] = 12;
      Buffer[9] = 0;    // start at 0
      Buffer[10] = 0;
      Buffer[11] = 0;
      Buffer[6] = (BufferSize >> 16) & 255;    // stop at BufferSize
      Buffer[7] = (BufferSize >> 8) & 255;
      Buffer[8] = BufferSize & 255;

      len = 12;

       s = XportTxData ((char*)Buffer ,&len);

       printf("Command 12 done with code=%d\n", s);

     for (b=0; b<NumBlocks; b++)
     {

//    setup command 10

      Buffer[0] = 0xc0;
      Buffer[1] = 0;
      Buffer[2] = 0;
      Buffer[3] = 4;
      Buffer[4] = 0xd0;
      Buffer[5] = 10;
      Buffer[6] = 0;
      Buffer[7] = 0;

       len = 8;

       s = XportTxData ((char*)Buffer ,&len);

       if (s != 0) {
           printf("Done with code=%d\n", s);
       } else {
           if (verbose) printf("Received %d bytes\n",len);
       }

     }


#ifdef WIN32
             time(&Time.tv_sec);
#else
             gettimeofday(&Time, NULL);
#endif

    EndTime = Time.tv_sec;
    printf("End at %d\n", EndTime);

             diff = EndTime-InitialTime;
             printf ("obtained %d blocks after %d\n", NumBlocks, diff);

    if (diff > 0) {
        rate = ((NumBlocks*BufferSize)/diff)/1024;
        printf ("rate approx %d Kbytes/sec\n", rate);
    }

      exit(0);
}
